﻿using Roslyn.Compilers;
using Roslyn.Compilers.Common;
using Roslyn.Compilers.CSharp;
using Roslyn.Services;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace FindingDateTimeNow
{
	public sealed class ChangeNowToUtcNowCodeAction
		: ICodeAction
	{
		private IDocument document;
		private MemberAccessExpressionSyntax nowNode;

		public ChangeNowToUtcNowCodeAction(IDocument document, MemberAccessExpressionSyntax nowNode)
		{
			this.document = document;
			this.nowNode = nowNode;
		}

		public string Description
		{
			get { return "Change Now to UtcNow."; }
		}

		public CodeActionEdit GetEdit(CancellationToken cancellationToken)
		{
			var nameNode = this.nowNode.Name;
			var utcNowNode = Syntax.IdentifierName("UtcNow");
			var rootNode = this.document.GetSyntaxRoot(cancellationToken);
			var newRootNode = rootNode.ReplaceNode(nameNode, utcNowNode);
			return new CodeActionEdit(document.UpdateSyntaxRoot(newRootNode));
		}
	}
}
