using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading;
using Roslyn.Compilers;
using Roslyn.Compilers.Common;
using Roslyn.Compilers.CSharp;
using Roslyn.Services;
using Roslyn.Services.Editor;

namespace FindingDateTimeNow
{
	[ExportCodeIssueProvider("FindingDateTimeNow", LanguageNames.CSharp)]
	public sealed class FindingDateTimeNowCodeIssueProvider
		: ICodeIssueProvider
	{
		public IEnumerable<CodeIssue> GetIssues(
			IDocument document, CommonSyntaxNode node, CancellationToken cancellationToken)
		{
			var memberNode = node as MemberAccessExpressionSyntax;

			if (memberNode.OperatorToken.Kind == SyntaxKind.DotToken &&
				memberNode.Name.Identifier.ValueText == "Now")
			{
				var symbol = document.GetSemanticModel().GetSymbolInfo(memberNode.Name).Symbol;

				if (symbol != null &&
					symbol.ContainingType.ToDisplayString() ==
						Values.ExpectedContainingDateTimeTypeDisplayString &&
					symbol.ContainingAssembly.ToDisplayString().Contains(
						Values.ExpectedContainingAssemblyDisplayString))
				{
					return new [] { new CodeIssue(CodeIssueKind.Error,
						memberNode.Name.Span,
						"Do not use DateTime.Now",
						new ChangeNowToUtcNowCodeAction(document, memberNode))};
				}
			}

			return null;
		}

		public IEnumerable<CodeIssue> GetIssues(
			IDocument document, CommonSyntaxToken token, CancellationToken cancellationToken)
		{
			throw new NotImplementedException();
		}

		public IEnumerable<Type> SyntaxNodeTypes
		{
			get
			{
				return new [] { typeof(MemberAccessExpressionSyntax) };
			}
		}

		public IEnumerable<int> SyntaxTokenKinds
		{
			get
			{
				return null;
			}
		}
	}
}
