﻿using Roslyn.Compilers;
using Roslyn.Compilers.Common;
using Roslyn.Compilers.CSharp;
using Roslyn.Services;
using Roslyn.Services.Editor;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace FindingDateTimeNow
{
	[ExportCodeIssueProvider("FindingNewDateTime", LanguageNames.CSharp)]
	public sealed class FindingNewDateTimeCodeIssueProvider
		: ICodeIssueProvider
	{
		public IEnumerable<CodeIssue> GetIssues(
			IDocument document, CommonSyntaxToken token, CancellationToken cancellationToken)
		{
			throw new NotImplementedException();
		}

		public IEnumerable<CodeIssue> GetIssues(
			IDocument document, CommonSyntaxNode node, CancellationToken cancellationToken)
		{
			var creationNode = node as ObjectCreationExpressionSyntax;
			var creationNameNode = creationNode.Type as IdentifierNameSyntax;

			if (creationNameNode != null && creationNameNode.Identifier.ValueText == "DateTime")
			{
				var model = document.GetSemanticModel();
				var creationSymbol = model.GetSymbolInfo(creationNode).Symbol;

				if (creationSymbol != null &&
					creationSymbol.ContainingType.ToDisplayString() ==
						Values.ExpectedContainingDateTimeTypeDisplayString &&
					creationSymbol.ContainingAssembly.ToDisplayString().Contains(
						Values.ExpectedContainingAssemblyDisplayString))
				{
					var argument = FindingNewDateTimeCodeIssueProvider.GetInvalidArgument(
						creationNode, model);

					if (argument != null)
					{
						if (argument.Item2.Name == "Local" ||
							argument.Item2.Name == "Unspecified")
						{
							return new [] { new CodeIssue(CodeIssueKind.Error,
								argument.Item1.Span,
								"Do not use DateTimeKind.Local or DateTimeKind.Unspecified", 
								new ChangeDateTimeKindToUtcCodeAction(document, argument.Item1)) };
						}
					}
					else
					{
						return new [] { new CodeIssue(CodeIssueKind.Error,
							creationNode.Span,
							"You must use a DateTime constuctor that takes a DateTimeKind") };
					}
				}
			}

			return null;
		}

		private static Tuple<ArgumentSyntax, ISymbol> GetInvalidArgument(
			ObjectCreationExpressionSyntax creationToken, ISemanticModel model)
		{
			foreach (var argument in creationToken.ArgumentList.Arguments)
			{
				if (argument.Expression is MemberAccessExpressionSyntax)
				{
					var argumentSymbolNode = model.GetSymbolInfo(argument.Expression).Symbol;

					if (argumentSymbolNode.ContainingType.ToDisplayString() ==
						Values.ExpectedContainingDateTimeKindTypeDisplayString)
					{
						return new Tuple<ArgumentSyntax,ISymbol>(argument, argumentSymbolNode);
					}
				}
			}

			return null;
		}

		public IEnumerable<Type> SyntaxNodeTypes
		{
			get { return new [] { typeof(ObjectCreationExpressionSyntax) }; }
		}

		public IEnumerable<int> SyntaxTokenKinds
		{
			get { return null; }
		}
	}
}
